/*
 * (c) Copyright 2002, Hewlett-Packard Company, all rights reserved.
 * [See end of file]
 */

package com.hp.hpl.jena.rdf.query.parser;

import com.hp.hpl.jena.rdf.query.*;
import com.hp.hpl.jena.util.* ;

public class ConstraintExpr implements Constraint
{
    Expr expr ;

    ConstraintExpr(Expr _expr)
    {
        expr = _expr ;
    }

    public boolean isSatified(Query q, ResultBinding env)
    {
        if ( expr == null )
            // This expression is in error
            return false ;

        try {
            if ( q.getLog() != null )
                q.getLog().debug("Eval: "+expr.asInfixString()+" :: "+env) ;
            return expr.eval(q, env).getBoolean() ;
        }
        catch (EvalFailureException e) //Includes EvalTypeException
        {
            // Check all exceptions possible.
            //expr = null ;
            return false ;
        }
        catch (Exception e)
        {
            // Shouldn't happen
            QSys.unhandledException(e, "ConstraintExpr", "isSatified") ;
            return false ;
        }
    }

    // Used in printing a query - not for getting the string
    // value of a expression (which itself must be unquoted)
    public String toString()
    {
        return expr.asInfixString() ;
    }
}

/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/).
 *
 */
