/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: BDBException.java,v 1.2 2001/09/25 14:52:09 bwm Exp $
 */

package com.hp.hpl.mesa.rdf.jena.bdb;

import com.hp.hpl.mesa.rdf.jena.model.RDFException;

import java.lang.Integer;

/** A class of RDF Exceptions specific to the Berkeley DB implementation.
 * @author bwm
 * @version $Name:  $ $Revision: 1.2 $ $Date: 2001/09/25 14:52:09 $
 */
public class BDBException extends RDFException {
    
    protected static final int MINERRORCODE                   = 100;
    protected static final int MAXERRORCODE                   = 106;
    
    /** There is a problem with the Berkeley DB environment directory
     */
    public static final int BADENVDIR                         = 101;
    
    /** cannot create the Berkeley DB environment directory is not a directory
     */
    public static final int CREATEENVDIR                      = 102;
    
    /** database is already open
     */
    public static final int DBALREADYOPEN                     = 103;
    
    /** database structure corrupt
     */
    public static final int DBCORRUPT                         = 104;
    
    /** Bad magic number in database header
     */
    public static final int BADMAGIC                          = 105;
    
    /** database header corrupt
     */
    public static final int HEADERCORRUPT                     = 106;
   

    protected static final String[] errorMessage = 
        { "not used",
          "There is a problem with the Berkeley DB environement directory",
          "The Berkeley DB environment directory is not a directory",
          "An attempt has been made to reopen an already open database",
          "The database is corrupt",
          "Did not find the correct magic number in the database configuration",
          "The database header is corrupt"
        };
    
    /** Create a new RDF exception with the given error code.
     * @param errorCode The code number of the error which has occurred.
     */
    public BDBException(int errorCode) {
        if (MINERRORCODE<=errorCode && errorCode<=MAXERRORCODE) {
            this.errorCode = errorCode;
        } else {
            this.errorCode = RDFException.INVALIDERRORCODE;
            this.otherCode = errorCode;
        }
    }
    
    /** Encapsulate an exception in an RDFException.
     * @param e The exception to be encapsulated.
     */  
    
    public BDBException(Exception e) {
        super(e);
    }
    
    /** Create a new RDFException with a given error code and encapsulate another
     * exception.
     * @param errorCode The code number of the error which has occurred.
     * @param e The exception to be encapsulated.
     */
    public BDBException(int errorCode, Exception e) {
        nestedException = e;
        if (MINERRORCODE<=errorCode && errorCode<=MAXERRORCODE) {
            this.errorCode = errorCode;
        } else {
            this.errorCode = RDFException.INVALIDERRORCODE;
            this.otherCode = errorCode;
        }
    }
    
    /** Return an error message describing the exception.
     * @return the error message
     */
    public String toString() {
        String message;
        
        if (MINERRORCODE<=errorCode && errorCode <=MAXERRORCODE) {
            message = "RDF Error code:" 
                    + Integer.toString(errorCode)
                    + " " + errorMessage[errorCode - MINERRORCODE];
        } else {
            message = super.toString();
        }
        return message;
    }
}