/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ByteSerializer.java,v 1.2 2002/09/04 13:01:33 bwm Exp $
 */

package com.hp.hpl.mesa.rdf.jena.bdb;

import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.model.*;

import com.hp.hpl.jena.util.Log;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

/**
 *
 * @author  bwm
 * @version $Name:  $ $Revision: 1.2 $ $Date: 2002/09/04 13:01:33 $
 */
    public abstract class ByteSerializer {
    
        protected static final byte ANONRESOURCETYPE      = 1;
        protected static final byte URIRESOURCETYPE       = 2;
        protected static final byte WELLFORMEDLITERALTYPE = 3;
        protected static final byte LITERALTYPE           = 4;
        protected static final byte PROPERTYTYPE          = 5;
    
        protected static void toData(RDFNode n, DataOutputStream os)
          throws java.io.IOException {
            if (n instanceof Property) {
                Property p = (Property) n;
                os.writeByte(PROPERTYTYPE);
                os.writeUTF(p.getNameSpace());
                os.writeUTF(p.getLocalName());
            } else if (n instanceof Resource) {
                Resource r = (Resource) n;
                if (r.isAnon()) {
                    os.writeByte(ANONRESOURCETYPE);
                    try {
                        os.writeUTF(r.getId().toString());
                    } catch (RDFException e1) {
                        Log.severe("failed getId failed",
                                   "ByteSerializer",
                                   "toData",
                                   e1);
                        throw new RDFError(e1);
                    }
                } else {
                    os.writeByte(URIRESOURCETYPE);
                    os.writeUTF(r.getURI());
                }
            } else {
                Literal l = (Literal) n;
                if (l.getWellFormed()) {
                    os.writeByte(WELLFORMEDLITERALTYPE);
                } else {
                    os.writeByte(LITERALTYPE);
                }
                try {
                    // writeUTF can only handle 64Kbyte-1 or less strings - bummer
                    os.writeUTF(l.getString());
                    os.writeUTF(l.getLanguage());
                } catch (RDFException e2) {
                    Log.severe("failed to get literal",
                               "ByteSerializer",
                               "toData",
                               e2);
                    throw new RDFError(e2);
                }
            }
        }
        
        protected static byte[] toBytes(RDFNode n) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            DataOutputStream os = new DataOutputStream(bos);
            try {
                toData(n, os);
            } catch (java.io.IOException e1) {
                Log.severe("toData failed",
                           "ByteSerializer",
                           "toData",
                           e1);
                throw new RDFError(e1);
            }
                
            try {
                os.flush();
            } catch (Exception e2) {
                Log.severe("failed to flush bytestream",
                           "ByteSerializer", "toBytes", e2);
                throw new RDFError(e2);
            }
            return bos.toByteArray();
        }
    
        protected static RDFNode fromData(DataInputStream is, Model m) 
          throws java.io.IOException{
            try {
                byte type = is.readByte();
        
                switch (type) {
                    case ANONRESOURCETYPE:
                        return new ResourceImpl(new AnonId(is.readUTF()), m);
                    
                    case URIRESOURCETYPE:
                        return new ResourceImpl(is.readUTF(), m);
                    
                    case WELLFORMEDLITERALTYPE:
                        return new LiteralImpl(is.readUTF(),is.readUTF(), true);
                    
                    case LITERALTYPE:
                        return new LiteralImpl(is.readUTF(), is.readUTF());
                    
                    case PROPERTYTYPE:
                        return new PropertyImpl(is.readUTF(), is.readUTF(), m);
                        
                    default:
                        Log.severe("Internal Error, type = " + type,
                                   "ByteSerializer", "fromData");
                        throw new RDFError(
                                   "Internal Error in ByteSerializer.fromData");
                }
          } catch (RDFException e1) {
              Log.severe("Internal Error", "ByteSerializer", "fromData", e1);
              throw new RDFError(e1);
          } 
        }
    }
