/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: StoreBdb.java,v 1.2 2001/09/25 14:52:09 bwm Exp $
 */

package com.hp.hpl.mesa.rdf.jena.bdb;

import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.model.*;

import com.hp.hpl.jena.util.Log;

import com.sleepycat.db.Db;
import com.sleepycat.db.Dbc;
import com.sleepycat.db.DbEnv;
import com.sleepycat.db.Dbt;
import com.sleepycat.db.DbBtreeStat;
import com.sleepycat.db.DbMemoryException;
import com.sleepycat.db.DbException;
import com.sleepycat.db.DbTxn;


/**
 *
 * @author  bwm
 * @version 
 */
class StoreBdb extends _StoreBdb implements Store {     
    
/** A non-transactional Berkely DB store.
 * @param envdir The directory holding the Berkeley DB environment.
 * @param filename The name of the Berkeley DB file which will hold the
 * model.  Normally this is relative to the environment
 * directory.
 * @param m The model using the store.
 * @throws RDFException Generic RDF exception
 */    
    public StoreBdb(String envdir, String filename, Model m) 
      throws RDFException {
        super(envdir, filename, m);
    }
    
/** Add a statement to the store
 * @param s the statement to add
 * @throws RDFException generic RDF exception
 */    
    public synchronized void add(Statement s) throws RDFException {
        stmtAdded = false;
        add(s, null);
        if (stmtAdded) size++;
    }
    
/** Remove a statement from the store
 * @param s the statement to remove
 * @throws RDFException generic RDF exception
 */    
    public synchronized void remove(Statement s) throws RDFException {
        stmtDeleted = false;
        remove(s, null);
        if (stmtDeleted) size--;
    }
}