/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: StoreBdbF.java,v 1.2 2001/09/25 14:52:09 bwm Exp $
 */

package com.hp.hpl.mesa.rdf.jena.bdb;

import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.model.*;

/** A factory class for creating instances of StoreDb
 * @author bwm
 * @version $Name:  $ $Revision: 1.2 $ $Date: 2001/09/25 14:52:09 $
 */
public class StoreBdbF implements StoreFactory {
    String filename;
    String envdir;
    
/** A factory object which will create stores in the environment defined
 * by <code>envdir</code> and the file <code>filename</code>.
 *
 * <p>If the <code>envdir</code> is <code>null</code>, then the default
 * environment directory will be used.  This is currently <code>
 * jena.rdf.bdb</code>.  The default may be overwritten by setting the
 * value of the system property <code>com.hp.hpl.mesa.rdf.jena.bdbenv</code>
 * </p>
 * @param envdir The name of the directory holding the Berkeley DB
 * environment in which the store will be created
 * @param filename the name of the file in which the store will be created.
 */    
    public StoreBdbF(String envdir, String filename) {
        this.envdir = envdir;
        this.filename = filename;
    }
    
/** Return a new store, properly configured.
 * @param model the model to which the store will be attached.
 * @throws RDFException generic RDF exception
 * @return the newly created store
 */    
    public Store createStore(Model model) throws RDFException {
        return new StoreBdb(envdir, filename, model);
    }
}