/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ResourceImpl.java
 *
 * Created on 03 August 2000, 13:45
 */

package com.hp.hpl.mesa.rdf.jena.common;

import com.hp.hpl.mesa.rdf.jena.model.*;

import java.rmi.server.UID;
import java.util.Random;

/** An implementation of Resource.
 *
 * @author  bwm
 * @version  Release='$Name:  $' Revision='$Revision: 1.7 $' Date='$Date: 2002/01/05 11:01:57 $'
 */
public class ResourceImpl implements Resource, ResourceI {
    protected String     nameSpace = null;
    protected String     localName = null;
    protected String     uri       = null;    
    protected Model      model     = null;
    protected AnonId     id        = null;
    
    /** Creates new ResourceImpl */
    public ResourceImpl() {
        assignId();
    }
    
    public ResourceImpl(String uri) {
        this.uri = uri;
        if (this.uri == null) {
            assignId();
        } else { 
            splitNameSpace();
        }
    }
    
    public ResourceImpl(String nameSpace, String localName) {
        this.nameSpace = nameSpace;
        this.localName = localName;
        this.uri = nameSpace + localName;
    }
    
    public ResourceImpl(Model m) {
        model = m;
        assignId();
    }
    
    public ResourceImpl(AnonId id) {
        this.id = id;
    }
    
    public ResourceImpl(AnonId id, Model m) {
        model = m;
        this.id = id;
    }
    
    public ResourceImpl(String uri, Model m) {
        this.uri = uri;
        this.model = m;
        if (this.uri == null) {
            assignId();
        } else {
            splitNameSpace();
        }
    }
    
    public ResourceImpl(Resource r, Model m) {
        uri = r.getURI();
        try {
            id = r.getId();
        } catch (RDFException e) {}
        nameSpace = r.getNameSpace();
        localName = r.getLocalName();
        model = m;
    }
    
    public ResourceImpl(String nameSpace, String localName, Model m) {
        this.nameSpace = nameSpace;
        this.localName = localName;
        uri = nameSpace + localName;
        model = m;
    }
    
    protected void assignId() {
        id = new AnonId();
    }    
    
    protected void splitNameSpace() {
        int splitPoint = Util.splitNamespace(uri);
        if (splitPoint == 0) {
            nameSpace = uri;
            localName = "";
        } else {
            nameSpace = uri.substring(0, splitPoint);
            localName = uri.substring(splitPoint);
        }
    }
    
    public AnonId getId() {
        return id;
    }
    
    public String  getURI() {
        return uri;
    }
    
    public String getNameSpace() {
        return nameSpace;
    }
    
    public String getLocalName() {
        return localName;
    }
    
    public String  toString() {
        if (uri == null) {
          return "anon:" + id;
        } else {
            return getURI();
        }
    }
    
    public boolean equals(Object o) {
        if (o instanceof Resource) {
            if (uri == null) {
                Resource r = ((ResourceI) o).getEmbeddedResource();
                if (r instanceof ResourceImpl) {
                    return id.equals(((ResourceImpl)r).id);
                } else {
                    return false;
                }
            }
            return uri.equals(((Resource) o).getURI());
        } else {
            return false;
        }
    }
        
    public int hashCode() {
        if (isAnon()) {
            return id.hashCode();
        } else {
            return uri.hashCode();
        }
    }
    
    public boolean isAnon() {
        return (uri == null);
    }
    
    public Statement getProperty(Property p) throws RDFException {
        if (model != null) {
            return model.getProperty(this, p);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    
    public StmtIterator listProperties(Property p) throws RDFException {
        if (model != null) {
            return model.listStatements(
                                     new SelectorImpl(this, p, (RDFNode) null));
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }        
    }
    
    public StmtIterator listProperties() throws RDFException {
        if (model != null) {
            return model.listStatements(
                                  new SelectorImpl(this, null, (RDFNode) null));
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    
    public Resource addProperty(Property p, boolean o) throws RDFException {
        if (model != null) {
            model.add(this, p, o);
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource addProperty(Property p, long o) throws RDFException {
        if (model != null) {
            model.add(this, p, o);
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource addProperty(Property p, char o) throws RDFException {
        if (model != null) {
            model.add(this, p, o);
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource addProperty(Property p, float o) throws RDFException {
        if (model != null) {
            model.add(this, p, o);
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource addProperty(Property p, double o) throws RDFException {
        if (model != null) {
            model.add(this, p, o);
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource addProperty(Property p, String o) throws RDFException {
        if (model != null) {
            model.add(this, p, o);
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource addProperty(Property p, String o, String l)
      throws RDFException {
        if (model != null) {
            model.add(this, p, o, l);
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource addProperty(Property p, Object o) throws RDFException {
        if (model != null) {
            model.add(this, p, o);
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource addProperty(Property p, RDFNode o) throws RDFException {
        if (model != null) {
            model.add(this, p, o);
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    
    public boolean hasProperty(Property p) throws RDFException {
        if (model != null) {
            return model.contains(this, p);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public boolean hasProperty(Property p, boolean o) throws RDFException {
        if (model != null) {
            return model.contains(this, p, o);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public boolean hasProperty(Property p, long o) throws RDFException {
        if (model != null) {
            return model.contains(this, p, o);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public boolean hasProperty(Property p, char o) throws RDFException {
        if (model != null) {
            return model.contains(this, p, o);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public boolean hasProperty(Property p, float o) throws RDFException {
        if (model != null) {
            return model.contains(this, p, o);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public boolean hasProperty(Property p, double o) throws RDFException {
        if (model != null) {
            return model.contains(this, p, o);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public boolean hasProperty(Property p, String o) throws RDFException {
        if (model != null) {
            return model.contains(this, p, o);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public boolean hasProperty(Property p, String o, String l)
      throws RDFException {
        if (model != null) {
            return model.contains(this, p, o, l);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public boolean hasProperty(Property p, Object o) throws RDFException {
        if (model != null) {
            return model.contains(this, p, o);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public boolean hasProperty(Property p, RDFNode o) throws RDFException {
        if (model != null) {
            return model.contains(this, p, o);
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    
    public Resource removeProperties() throws RDFException {
        if (model != null) {
            StmtIterator iter = null;
            iter = model.listStatements(
                                 new SelectorImpl(this, null, (RDFNode) null));
            while (iter.hasNext()) {
                iter.next();
                iter.remove();
            }
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    
    public Resource begin() throws RDFException {
        if (model != null) {
            model.begin();
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource abort() throws RDFException {
        if (model != null) {
            model.abort();
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    public Resource commit() throws RDFException {
        if (model != null) {
            model.commit();
            return this;
        } else {
            throw new RDFException(RDFException.NOTRELATEDTOMODEL);
        }
    }
    
    public Resource port(Model m) throws RDFException {
        if (model == m) {
            return this;
        } else {
            return new ResourceImpl(this, m);
        }
    }
    public Model getModel() {
        return model;
    }
    public Resource getEmbeddedResource() {
        return this;
    }
}