/*
 *  (c) Copyright Hewlett-Packard Company 2001 
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * StoreCom.java
 *
 * Created on 10 Mar 2001, 12:53
 */

package com.hp.hpl.mesa.rdf.jena.common;

import com.hp.hpl.mesa.rdf.jena.model.*;

/**
 *
 * @author  bwm
 * @version $Version$
 */
public abstract class StoreCom extends Object implements Store {
    

    /** Creates new Store */
    public StoreCom() {
    } 
    
    public Resource createResource(Model m) throws RDFException {
        return new ResourceImpl(m);
    }
    
    public Resource createResource(String uri,Model m) throws RDFException {
        return new ResourceImpl(uri, m);
    }
    
    public Property createProperty(String uri, Model m) throws RDFException {
          return new PropertyImpl(uri, m);
    }
    
    public Property createProperty(String namespace,String localPart,Model m)
      throws RDFException {
          return new PropertyImpl(namespace, localPart, m);
    }
    
    public Statement createStatement(Resource s, Property p, RDFNode o, Model m)
      throws RDFException {
          return new StatementImpl(s, p, o, m);
    }
    
    public Bag createBag(Model m) throws RDFException {
        return new BagImpl(m);
    }
    
    public Bag createBag(String uri, Model m) throws RDFException {
        return new BagImpl(uri, m);
    }
    
    public Alt createAlt(Model m) throws RDFException {
        return new AltImpl(m);
    }
    
    public Alt createAlt(String uri,Model m) throws RDFException {
        return new AltImpl(uri, m);
    }
    
    public Seq createSeq(Model m) throws RDFException {
        return new SeqImpl(m);
    }
    
    public Seq createSeq(String uri,Model m) throws RDFException {
        return new SeqImpl(uri, m);
    }
    
    public Resource getResource(String uri, Model m) throws RDFException {
        return new ResourceImpl(uri, m);
    }    
    
    public Property getProperty(String uri, Model m)
      throws RDFException {
          return getProperty(new PropertyImpl(uri, m));
    }   
    
    public Property getProperty(String namespace, String localPart, Model m)
      throws RDFException {
        return getProperty(new PropertyImpl(namespace, localPart, m));
    }
    
    public Statement getStatement(Resource s, Property p, RDFNode o, Model m) 
      throws RDFException {
          return new StatementImpl(s, p, o, m);
    }
    
    public Property getProperty(Property p) throws RDFException {
        return p;
    }
    
    public Bag getBag(String uri, Model m) throws RDFException {
        return getBag(getResource(uri, m), m);
    }
    
    public Bag getBag(Resource r, Model m) throws RDFException {
        if (r instanceof Bag) {
            return (Bag) r;
        } else {
            return new BagImpl(r, m);
        }
    }
    
    public Alt getAlt(String uri, Model m) throws RDFException {
        return getAlt(getResource(uri, m), m);
    }
    
    public Alt getAlt(Resource r, Model m) throws RDFException {
        if (r instanceof Alt) {
            return (Alt) r;
        } else {
            return new AltImpl(r, m);
        }
    }
    
    public Seq getSeq(String uri, Model m) throws RDFException {
        return getSeq(getResource(uri, m), m);
    }
    
    public Seq getSeq(Resource r, Model m) throws RDFException {
        if (r instanceof Seq) {
            return (Seq) r;
        } else {
            return new SeqImpl(r, m);
        }
    }
    
    public void close() {}
}