/*****************************************************************************
 * Source code information
 * -----------------------
 * Original author    Jeremy Carroll, HP Labs Bristol
 * Author email       jjc@hpl.hp.com
 * Package            Jena
 * Created            10 Nov 2000
 * Filename           $RCSfile: PrettyWriterTest.java,v $
 * Revision           $Revision: 1.1 $
 * Release status     Preview-release $State: Exp $
 *
 * Last modified on   $Date: 2002/07/02 00:07:56 $
 *               by   $Author: jeremy_carroll $
 *
 * (c) Copyright Hewlett-Packard Company 2002
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

// Package
///////////////
package com.hp.hpl.mesa.rdf.jena.common.prettywriter.test;


// Imports
///////////////
import junit.framework.*;

import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.mem.*;

import java.io.*;


/**
 * JUnit regression tests for the Jena DAML model.
 *
 * @author Jeremy Carroll
 * @version CVS info: $Id: PrettyWriterTest.java,v 1.1 2002/07/02 00:07:56 jeremy_carroll Exp $,
 */
public class PrettyWriterTest
    extends TestCase
{
   

    /**
     * Constructor requires that all tests be named
     *
     * @param name The name of this test
     */
    public PrettyWriterTest( String name ) {
        super( name );
    }



    // External signature methods
    //////////////////////////////////

    /**
     * Answer a suite of all the tests defined here
     */
    public static Test suite() {
        TestSuite suite = new TestSuite();

        // add all the tests defined in this class to the suite
        /* */ suite.addTest( new PrettyWriterTest( "testAnonDamlClass" ) );    /* */

        return suite;
    }



    // Internal implementation methods
    //////////////////////////////////




    // Test cases
    /////////////

    /**
     * Test the various pathways through loading the ontology from a source document.
     */
    public void testAnonDamlClass()
        throws RDFException, IOException
    {
    	String filename = "file:modules/rdf/regression/testWriterAndReader/prettywriter.daml";
    	Model m = new ModelMem();
    	m.read(filename);
    	StringWriter sw = new StringWriter();
    	m.write(sw,"RDF/XML-ABBREV",filename);
    	sw.close();
    	Model m2 = new ModelMem();
    	m2.read(new StringReader(sw.toString()),filename);
    	assertEquals(m,m2);
    	
    }



}


