/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: TestWriterXMLBaseProp.java,v 1.2 2002/08/03 05:43:49 jeremy_carroll Exp $
 */

package com.hp.hpl.mesa.rdf.jena.common.regression;

import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.common.regression.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.io.FileReader;
import java.io.*;

/** 
 * @author bwm
 * @version $Name:  $ $Revision: 1.2 $ $Date: 2002/08/03 05:43:49 $
 */
public class TestWriterXMLBaseProp extends TestCase {
    
    protected Model model = null;
    String base1 = "http://example/foobar";
    String base2 = "http://example/barfoo";
    
    public TestWriterXMLBaseProp(String name) {
        super(name);
    }
    
    public static Test suite() {
        return new TestSuite(TestWriterXMLBaseProp.class); 
    }
    
    public void testBasicXMLBase() {
        doTest("RDF/XML");
    }
    /* Doesn't yet pass.
    public void testBasicPropertyURI() {
        doBadPropTest("RDF/XML");
    }
    */
    
    public void testAbbrevXMLBase() {
        doTest("RDF/XML-ABBREV");
    }
     
    public void testAbbrevPropertyURI() {
        doBadPropTest("RDF/XML-ABBREV");
    }

    
    public void doTest(String lang) {
        File file = null;
        try {
            model = new ModelMem();
            RDFWriter writer = model.getWriter(lang);
            
            String oldvalue = (String) writer.setProperty("xmlbase", base1);
            assertTrue("xmlbase valued non-null",oldvalue == null);
            
            oldvalue = (String) writer.setProperty("xmlbase", base2);
            assertEquals("xmlbase valued incorrect.",base1, oldvalue);
          
            file = File.createTempFile("rdf", ".xml");
            file.deleteOnExit();
        
            Writer fwriter = 
              new OutputStreamWriter(new FileOutputStream(file),"utf-8");
            
            writer.write(model, fwriter , "");
            fwriter.close();
        
            FileReader freader = new FileReader(file);
        
            String rdf = "";
            int i = freader.read();
            while (i>=0) {
                rdf = rdf + (char) i;
                i = freader.read();
            }
            freader.close();
            // this code makes some assumptions about the formating
            // which it shouldn't, but I haven't time to write the
            // sax code to do it properly
            // These assumptions look fine to me - jjc.
            if ( rdf.indexOf("xml:base=\"" + base2 + "\"") < 0 &&
                 rdf.indexOf("xml:base='"  + base2 + "'") < 0) {
                assertTrue("xml:base not found",false);
            }
        } catch (Exception e) {
            e.printStackTrace(System.out);
            assertTrue(e.toString(),false);
        } finally {
        	if (file != null)
                 file.delete();
        }
    }
        
    void doBadPropTest(String lang) {
    	try {
    	Model m = new ModelMem();
    	m.add(m.createResource(),
    	      m.createProperty("http://example/","foo#"),
    	      "foo");
    	File file = File.createTempFile("rdf", ".xml");
            file.deleteOnExit();
        
        FileWriter fwriter = new FileWriter(file);
        try {
            m.write(fwriter,lang);
        // throw new RDFException(RDFException.INVALIDPROPERTYURI);
             fail("Writer did not detect bad property URI");
        }
        catch (RDFException rdfe) {
        	// This loop here really shouldn't be necessary.
        	// When oh when, will Brian finally get round to 
        	//  - making RDFException runtime
        	//  - dropping NestedExceptions altogther.
        	while ( rdfe.getErrorCode() == rdfe.NESTEDEXCEPTION 
        	         && rdfe.getNestedException() instanceof RDFException )
        	         rdfe = (RDFException)rdfe.getNestedException();
        	assertEquals("Inappropriate exception: "+ rdfe.getMessage(),rdfe.getErrorCode(),rdfe.INVALIDPROPERTYURI);
        }
        fwriter.close();
        file.delete();
    	} catch (Exception e) {
    		e.printStackTrace();
    		fail("Unexpected exception: "+e.toString());
    	}
    }

}