/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: testWriterAndReader.java,v 1.8 2002/09/07 12:42:13 bwm Exp $
 */

package com.hp.hpl.mesa.rdf.jena.common.regression;

import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.common.*;

import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;


import com.hp.hpl.mesa.rdf.jena.common.prettywriter.Filter;


import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import java.util.*;

/**
 * This will test any Writer and Reader pair.
 * It writes out a random model, and reads it back in.
 * The test fails if the models are not 'the same'.
 * Quite what 'the same' means is debatable.
 * @author  jjc
 
 * @version  Release='$Name:  $' Revision='$Revision: 1.8 $' Date='$Date: 2002/09/07 12:42:13 $'
 */
public class testWriterAndReader implements RDFErrorHandler {
    static private boolean showProgress = false;
    static private boolean keepFiles = false;
    static private boolean errorDetail = false;
    static private int firstTest = 5;
    static private int lastTest = 9;
    static private int repetitionsJ = 6;
    
    private String test = " test testWriterAndReader initialization";
    private int n = 0;
    
    protected static void doTest(Model m1,Model m2,Model m3,Model m4) {
        if (showProgress)
            System.out.println("Each + below corresponds to 80 tests.");
        (new testWriterAndReader()).test(m1,m2,m3,m4);
    }
    
    public void test(Model m1, Model m2,Model m3,Model m4) {
        try {
            test("N-TRIPLE",25,1,1,m1,m2,m3,m4, null, null);
            test("RDF/XML-ABBREV",35,1,1,m1,m2,m3,m4, null, null);
            test("RDF/XML",35,1,1,m1,m2,m3,m4, null, null);
            
            {
                String[] propNames = {"longId"};
                Object[] propVals  = { new Boolean(true)};
                test("RDF/XML",35,1,1,m1,m2,m3,m4, propNames, propVals);
            }
        } catch (Exception e) {
            error( e );
        }
    }
    /*
    public void  test(String rwType,  Model m1, Model m2,Model m3,Model m4) throws RDFException {
        test(rwType,0,m1,m2,m3,m4);
    }
    public void  randomTest(String rwLang,  Model m1, Model m2,Model m3,Model m4) throws RDFException {
        test(rwLang,(int)System.currentTimeMillis(),m1,m2,m3,m4);
    }
    */
    static final String baseUris[] = {
        "http://foo.com/Hello",
        "http://foo.com/Hello",
        "http://daml.umbc.edu/ontologies/calendar-ont",
        "http://www.daml.org/2001/03/daml+oil-ex"
    };
    public void test(String rwLang, int seed, int iiiMax, int jjjMax, Model m1,
                      Model m2,Model m3,Model m4, 
                      String[] wopName,  Object[] wopVal)  // properties to
                                                           // set on the writer
                                                          throws RDFException {
        
        test = "testWriterAndReader lang=" + rwLang + " seed="+seed;
        String  filebase = "modules/rdf/regression/testWriterAndReader/";
        if (showProgress)
            System.out.println("Beginning " + test);
        seenError = new BitSet();
        Random random = new Random(seed);
        File tmpFile1;
        MakeAnon makeA = new MakeAnon(rwLang);
        RDFReader rdfRdr = m1.getReader(rwLang);
        RDFWriter rdfWtr = m1.getWriter(rwLang);
        
        // set any writer options
        
        if (wopName != null) {
            for (int i=0; i<wopName.length; i++) {
                rdfWtr.setProperty(wopName[i], wopVal[i]);
            }
        }
        
        rdfRdr.setErrorHandler(this);
        rdfWtr.setErrorHandler(this);
        for ( int iii=0; iii<iiiMax;iii++) {
            for ( int jjj=0; jjj<jjjMax; jjj++) {
                try {
                    for (int i=firstTest;i<=lastTest;i++) {
                        n = 1000 * i;
                        //  System.out.println(tmpFile1.toString());
                        n++; String fileName = "t" + (i*1000) + ".rdf";
                        n++; empty(m1);
                        n++; String baseUriRead;
                        if ( i < baseUris.length )
                            baseUriRead = baseUris[i];
                        else
                            baseUriRead = "http://foo.com/Hello";
                        n++;
                        Reader rdr = new ResourceReader(filebase+fileName);
                        n++; m1.read(rdr,baseUriRead);
                        n++; rdr.close();
                        boolean problem = false;
                        for ( int j=0;j<repetitionsJ;j++) {
                            // n=0
                            n = 1000 *i + j* 100 + 20;
                            tmpFile1 = File.createTempFile("j"+n+"t",".txt");
                            n++;
                            String baseUriWrite =
                            j%2==0
                            ? baseUriRead
                            : "http://bar.com/irrelevant";
                            int cn = (int)m1.size();
                            n++; if ( (j%2)==0 && j>0 ) prune(m1,random,1+cn/10);
                            n++; if ( (j%2)==0 && j>0 ) expand(m1,random,1+cn/10);
                            // n = 23
                            n++; PrintWriter pw = new java.io.PrintWriter(new FileWriter(tmpFile1));
                            //    System.out.println("Writing " + tmpFile1.getName());
                            n++; rdfWtr.write( m1, pw, baseUriWrite);
                           // n++; pw.print("<!-- " + n + " -->");
                            n++; pw.close();
                            n++; empty(m2);
                            n++; InputStream in = new FileInputStream(tmpFile1);
                            // n = 28
                            n++; rdr = new InputStreamReader(in);
                            n++; rdfRdr.read(m2, rdr, baseUriWrite);
                            n++; in.close();
                            n++; rdr.close();
                            // It is possible that when they were read in
                            // some of the resources had an anonymous looking
                            // URI - in which case we need to use the full-works,
                            // since their variants in m2 will be made anonymous.
                            n++; Model s1 = makeA.makeAnon(m1,baseUriWrite,m3);
                            int cnt1 = makeA.lastCnt();
                            // n = 33
                            n++; Model s2 = makeA.makeAnon(m2,baseUriWrite,m4);
                            int cnt2 = makeA.lastCnt();
                            if (cnt1>50)
                              System.out.println(cnt1);
                            n++; if (cnt1!=cnt2 || (!s1.equals(s2))) {
                                problem = true;
                                error();
                                pw = new PrintWriter(new FileWriter(File.createTempFile("j" + n +"b",".txt")));
                                m1.write(pw);
                                pw.close();
                                pw = new PrintWriter(new FileWriter(File.createTempFile("j" + n + "b",".txt")));
                                m2.write(pw);
                                pw.close();
                                pw = new PrintWriter(new FileWriter(File.createTempFile("j" + n + "b",".txt")));
                                s1.write(pw);
                                pw.close();
                                pw = new PrintWriter(new FileWriter(File.createTempFile("j" + n + "b",".txt")));
                                s2.write(pw);
                                pw.close();
                                if ( errorDetail ) {
                                    int diff = (int)(s1.size() - s2.size());
                                    System.out.print("Diff is " + diff);
                                    int seenCnt = 0;
                                    Model big, small;
                                    if ( diff >= 0 ) {
                                        big = s1;
                                        small = s2;
                                        
                                    } else {
                                        big = s2;
                                        small = s1;
                                    }
                                    StmtIterator ss = big.listStatements();
                                    try {
                                        while ( seenCnt < 10 && ss.hasNext() ) {
                                            Statement stmt = ss.next();
                                            if (!(small.contains(stmt)
                                                  || stmt.getSubject().isAnon())
                                                ) {
                                                RDFNode obj = stmt.getObject();
                                                if (!((obj instanceof Resource)
                                                && ((Resource)obj).isAnon())) {
                                                    seenCnt++;
                                                    System.out.println(stmt);
                                                }
                                            }
                                        }
                                    }
                                    finally {
                                        ss.close();
                                    }
                                }
                                
                            }
                            
                            if ( ! (keepFiles||problem) ) {
                                n++; tmpFile1.delete();
                            }
                            
                        }
                        
                    }
                } catch (Exception e) {
                    error( e);
                }
                if (showProgress) {
                System.out.print("+");
                System.out.flush();
                }
            }
            if (showProgress)
                System.out.println();
        }
        if (showProgress)
            System.out.println("End of " + test);
    }
    /**Deletes cnt edges from m chosen by random.
     * @param cnt The number of statements to delete.
     * @param m A model with more than cnt statements.
     */
    private void prune(Model m,Random random,int cnt) throws RDFException {
        //    System.out.println("Pruning from " + (int)m.size() + " by " + cnt );
        Statement die[] = new Statement[cnt];
        int sz = (int)m.size();
        StmtIterator ss = m.listStatements();
        try {
            for (int i=0;i<cnt;i++)
                die[i] = ss.next();
            while ( ss.hasNext() ) {
                int ix = random.nextInt(sz);
                if ( ix < cnt )
                    die[ix] = ss.next();
            }
        }
        finally {
            ss.close();
        }
        for (int i=0;i<cnt;i++)
            m.remove(die[i]);
        //    System.out.println("Reduced to " + (int)m.size()  );
    }
    /**
     *  Adds cnt edges to m chosen by random.
     *
     * @param cnt The number of statements to add.
     * @param m A model with more than cnt statements.
     */
    private void expand(Model m,Random random,int cnt) throws RDFException {
        // System.out.println("Expanding from " + (int)m.size() + " by " + cnt );
        Resource subject[] = new Resource[cnt];
        Property predicate[] = new Property[cnt];
        RDFNode object[] = new RDFNode[cnt];
        int sz = (int)m.size();
        StmtIterator ss = m.listStatements();
        try {
            for (int i=0;i<cnt;i++) {
                Statement s = ss.next();
                subject[i] = s.getSubject();
                predicate[i] = s.getPredicate();
                object[i] = s.getObject();
            }
            while ( ss.hasNext() ) {
                Statement s = ss.next();
                Resource subj = s.getSubject();
                RDFNode obj = s.getObject();
                int ix = random.nextInt(sz);
                if ( ix < cnt )
                    subject[ix] = subj;
                ix = random.nextInt(sz);
                if ( ix < cnt )
                    object[ix] = subj;
                ix = random.nextInt(sz);
                if ( ix < cnt )
                    predicate[ix] = s.getPredicate();
                ix = random.nextInt(sz);
                if ( ix < cnt )
                    object[ix] = obj;
                if ( obj instanceof Resource ) {
                    ix = random.nextInt(sz);
                    if ( ix < cnt )
                        subject[ix] = (Resource)obj;
                }
            }
        }
        finally {
            ss.close();
        }
        for (int i=0;i<cnt;i++)
            m.add(subject[i],predicate[i],object[i]);
        //   System.out.println("Expanded to " + (int)m.size()  );
    }
    BitSet seenError = new BitSet();
    private void error() {
        if (!seenError.get(n))
            System.out.println(test + ": failed test " + Integer.toString(n));
        seenError.set(n);
    }
    public void error( Exception e) {
        if (!seenError.get(n))
            System.out.println("Test Failed: "
            + test + " "
            + n  + " "
            + e.toString());
        seenError.set(n);
    }
    public boolean getErrors() {
        return !seenError.equals(new BitSet());
    }
    
/** report a warning
 * @param e an exception representing the error
 */
    public void warning(Exception e) {
    }
    
    
/** report a catastrophic error.  Must not return.
 * @param e an exception representing the error
 * @throws RDFError a generic RDF exception
 */
    public void fatalError(Exception e) {
        error(e);
        throw new RDFError(e);
    }
    static public void empty(Model m) throws RDFException {
        StmtIterator iter = m.listStatements();
        while (iter.hasNext()) {
            iter.next();
            iter.remove();
        }
    }
   static ModelMem load(String fName) throws RDFException, IOException {
        ModelMem m2 = new ModelMem();
        ModelMem m3 = new ModelMem();
        RDFReader rdfRdr = m2.getReader();
        InputStream in = new FileInputStream("c:\\"+fName);
        // n = 28
        Reader rdr = new InputStreamReader(in);
        rdfRdr.read(m2, rdr, "http://should.not.see.me/");
        in.close();
        rdr.close();
        MakeAnon makeA = new MakeAnon("RDF/XML" );
        // It is possible that when they were read in
        // some of the resources had an anonymous looking
        // URI - in which case we need to use the full-works,
        // since their variants in m2 will be made anonymous.
        return (ModelMem)makeA.makeAnon(m2,"http://should.not.see.me/",m3);
    }
    // RUN THIS TEST ONLY
    static public void main(String args[]) throws RDFException, IOException{
        
        showProgress = true;
     //   firstTest = 1;
     //   repetitionsJ = 10;
        testWriterAndReader tester = new testWriterAndReader();
        Model m1 = new com.hp.hpl.mesa.rdf.jena.mem.ModelMem();
        Model m2 = new com.hp.hpl.mesa.rdf.jena.mem.ModelMem();
        Model m3 = new com.hp.hpl.mesa.rdf.jena.mem.ModelMem();
        Model m4 = new com.hp.hpl.mesa.rdf.jena.mem.ModelMem();
        try {
        //    tester.test("N-TRIPLE",25,1,2,m1,m2,m3,m4);
        //    tester.test("RDF/XML-ABBREV",35,3,20,m1,m2,m3,m4);
           tester.test("N-TRIPLE",25,1,1,m1,m2,m3,m4, null, null);
           tester.test("RDF/XML-ABBREV",35,1,1,m1,m2,m3,m4, null, null);
           tester.test("RDF/XML",35,1,1,m1,m2,m3,m4, null, null);
            
            {
                String[] propNames = {"longId"};
                Object[] propVals  = { new Boolean(true)};
                tester.test("RDF/XML",35,1,1,m1,m2,m3,m4, propNames, propVals);
            }
        } catch (Exception e) {
            tester.error( e );
        }
        
    }
    
}


 class MakeAnon {
        Filter isAnon;
        Map    mapping;
        Model rslt;
        String baseNameSpace;
        Set   anonymous;
        MakeAnon(String lang) {
           
            if ( lang.equals("RDF/XML-ABBREV") ) {
                isAnon = new Filter() {
                    public boolean accept(Object o) {
                        String s = (String)o;
                        if ( s.startsWith("RDFAnon") ) {
                            return true; // More care is needed really.
                        }
                        return false;
                    }
                };
            } else if ( lang.equals("RDF/XML") ) {
                isAnon = new Filter() {
                    public boolean accept(Object o) {
                        String s = (String)o;
                        if ( s.startsWith("A") ) {
                            try {
                            Integer.parseInt(s.substring(1));
                            return true;
                            }
                            catch (NumberFormatException e) {
                            }
                        }
                        return false;
                    }
                };
                
            } else if ( lang.equals("N-TRIPLE") ) {
                isAnon = new Filter() {
                    public boolean accept(Object o) {
                        return false;
                    }
                };
            } else {
               throw new IllegalArgumentException("Unsupported language: " + lang);
            }
        }
        private void addStatements(StmtIterator ss) throws RDFException {
            try {
                while (ss.hasNext()) {
                    rslt.add(mapAnon(ss.next()));
                }
            }
            finally {
                ss.close();
            }
        }
        Model makeAnon(Model m, String baseNS, Model mR) throws RDFException {
            rslt = mR;
            anonymous = new HashSet();
            testWriterAndReader.empty(rslt);
            mapping = new HashMap();
            baseNameSpace = baseNS;
            addStatements(m.listStatements());
            return rslt;
        }
        private Statement mapAnon(Statement s) throws RDFException{
            Resource subj = s.getSubject();
            RDFNode obj = s.getObject();
            Resource nSubj;
            RDFNode nObj;
            nSubj = map(subj);
            if (obj instanceof Resource ) {
                nObj = map((Resource)obj);
            } else {
                nObj = obj;
            }
            if (nSubj!=subj || nObj != obj)
                return new StatementImpl(nSubj,s.getPredicate(),nObj);
            else
                return s;
        }
        private Resource map(Resource r) throws RDFException {
            if ( r.isAnon() ) {
                anonymous.add(r);
                return r;
            }
            String uri = r.getURI();
            /*
            if (uri.endsWith("Anon1690")) {
                System.out.println("DebugMe");
            }
             **/
            int hash = uri.lastIndexOf('#');
            if ( hash== -1)
                return r;
            if ( hash != baseNameSpace.length() || (!uri.startsWith(baseNameSpace)) )
                return r;
            String localName = uri.substring(hash+1);
            if ( isAnon.accept(localName) ) {
                Resource m = (Resource)mapping.get(r);
                if ( m == null ) {
                    m = rslt.createResource();
                    anonymous.add(m);
                    mapping.put(r,m);
                }
                return m;
            } else {
                return r;
            }
        }
        int lastCnt() {
            return anonymous.size();
        }
    }