/*
 *  (c) Copyright Hewlett-Packard Company 2001 
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: testWriterInterface.java,v 1.8 2002/07/09 10:11:32 jeremy_carroll Exp $
 */

package com.hp.hpl.mesa.rdf.jena.common.regression;

import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.common.*;

import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.Writer;

/**
 *
 * @author  bwm
 * @version $Revision: 1.8 $
 */
public class testWriterInterface extends Object {
   
    
    protected static void doTest(Model m1) {
        (new testWriterInterface()).test(m1);
    }

    void test(Model m1) {

        String  test = "testWriterInterface";
        String  filebase = "modules/rdf/regression/" + test + "/";
     //   System.out.println("Beginning " + test);
        int n = 0;
        File file = null;
        Writer output = null;
        try {
        	
            n++; RDFWriter writer = m1.getWriter();
                 if (! (writer instanceof RDFXMLWriter)) error(test, n);
            
            n++; writer = m1.getWriter("RDF/XML");
                 if (! (writer instanceof RDFXMLWriter)) error(test, n);
            
            n++; writer = m1.getWriter("N-TRIPLE");
                 if (! (writer instanceof NTripleWriter)) error(test, n);
            
            n++; try {
                    m1.setWriterClassName("foobar", "");
                    writer = m1.getWriter("foobar");
                    error(test, n);
                 } catch (RDFException e) {
                     if (e.getErrorCode() != RDFException.NOWRITERFORLANG) {
                         error(test, n);
                     }
                 }
                 
            n++; m1.setWriterClassName("foobar", 
                                       Jena.PATH + ".common.RDFXMLWriter");
                 writer = m1.getWriter("foobar");
                 if (! (writer instanceof RDFXMLWriter)) error(test, n);
                 
            n++; file = File.createTempFile("~jena", ".rdf");
                 output = new PrintWriter(new FileWriter(file));
                 m1.write(output);
                 output.close();
                 output = null;
            
            n++; 
                 output = new PrintWriter(new FileWriter(file));
                 m1.write(output, "N-TRIPLE");
                 output.close();
                 output = null;
            
            n++; 
                 output = new PrintWriter(new FileWriter(file));
                m1.write(output, "N-TRIPLE", "");
                 output.close();
                 output = null;
            
        } catch (Exception e) {
            inError = true;
            ErrorHelper.logInternalError(" test " + test, n, e);
        } finally {
        	if ( output != null)
        	  try {
        	   output.close();
        	  }
        	  catch (Exception e) {
        	  }
        	if ( file != null )
                file.delete();
        }
     //   System.out.println("End of " + test);        
    }
    private boolean inError = false;
       
    protected void error(String test, int n) {
        System.out.println(test + ": failed test " + Integer.toString(n));
        inError = true;
    }
    
    public boolean getErrors() {
        return inError;
    }
    
}
