/*
 *  (c) Copyright Hewlett-Packard Company 2001 
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * StmtIterator.java
 *
 * Created on 28 July 2000, 13:44
 */

package com.hp.hpl.mesa.rdf.jena.model;

/**
 *
 * @author  bwm
 * @version  Release='$Name:  $' Revision='$Revision: 1.2 $' Date='$Date: 2001/09/25 14:52:10 $'
 */
import java.util.Iterator;



/** An iterator which should be closed after use
 *
 * <p>Some iterators take up resources which should be free'd as soon as
 * possible, i.e. without waiting for the garbage collector to call the
 * the finalizer.</p>
 *
 * <p>Implementors of this interface are expected to close automatically if
 * if the iterator iterates to completion, i.e. more() will return false.</p>
 *
 * @author bwm
 * @version Release='$Name:  $' Revision='$Revision: 1.2 $' Date='$Date: 2001/09/25 14:52:10 $'
 */
public interface ClosableIterator extends Iterator {
    
    /** Close the iterator
     */
    public void close();
}