/* -*-Java-*-
*******************************************************************
*
* File:         DBIDHash.java
* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/DBIDHash.java,v 1.2 2001/11/26 18:20:08 der Exp $
* Author:       Dave Reynolds, HPLabs, Bristol
* Created:      30/10/01
* Modified:     $Date: 2001/11/26 18:20:08 $
* Status:       Experimental (Do Not Distribute)
*
* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.
* [See end of file]
*********************************************************************/

//=======================================================================
// Package
package com.hp.hpl.mesa.rdf.jena.rdb;

//=======================================================================
// Imports
import com.hp.hpl.jena.util.Log;

//=======================================================================
/**
* Interface for database identifiers.
* Most RDF entities (resources, literals, statements) have an associated
* database index. These are cached using RDB-specific variants of the jena
* "impl" classes. This can avoid some redundant database lookup.
* <p>
* This variant uses 8 character strings to stort 64bit hash values. Uses this
* representation instead of longs because some of our databases don't directly
* support longs but everyone supports char arrays (?)
*
* @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>
* @version $Revision: 1.2 $ on $Date: 2001/11/26 18:20:08 $
*/

public class DBIDHash implements IDBID {

    /** The index */
    protected String m_dbid;

    /** constructor */
    public DBIDHash(char[] id) {
        m_dbid = new String(id);
    }

    /** constructor */
    public DBIDHash(byte[] id) {
        char[] chars = new char[id.length];
        for (int i = 0; i < id.length; i++) {
            chars[i] = (char)((char) id[i] & '\u00ff');
        }
        m_dbid = new String(chars);
    }

    /** constructor */
    public DBIDHash(String id) {
        m_dbid = id;
    }

    /** get the identifier as a String, fits calling signature of our generic sql interface. */
    public Object getID() {
        return m_dbid;
    }

    /** get first 64 bits of the hash id as a Long */
    public long getIDLong() {
        long res = 0;
        char[] b = new char[8];
        m_dbid.getChars(0, 8, b, 0);
        for (int i = 7; i >= 0; i--) {
            res <<= 8;
            res |= (long)b[i] & 0xffl;
        }
        return res;
    }

    /** Hash is based on the underlying object */
    public int hashCode() {
        return m_dbid.hashCode();
    }

    /** Equality is based on the underlying object */
    public boolean equals(Object obj) {
        if (obj instanceof IDBID) {
            return getID().equals(((IDBID)obj).getID());
        } else {
            return false;
        }
    }
}

/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

