/* -*-Java-*-
*******************************************************************
*
* File:         ICache.java
* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/ICache.java,v 1.1 2002/08/09 12:12:50 der Exp $
* Author:       Dave Reynolds, HPLabs, Bristol
* Created:      8/8/02
* Modified:     $Date: 2002/08/09 12:12:50 $
* Status:       Experimental (Do Not Distribute)
*
* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.
* [See end of file]
*********************************************************************/

//=======================================================================
// Package
package com.hp.hpl.mesa.rdf.jena.rdb;

//=======================================================================
/**
* Interface signature for cache implementations. Instances of this are
* used to cache resources and literals loaded from a database.
* <p>The type of the stored objects is left generic in case other caches
* are needed but could specialize it to RDFNodes.
*
* @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>
* @version $Revision: 1.1 $ on $Date: 2002/08/09 12:12:50 $
*/


public interface ICache {

    /**
     * Add an entry to the cache
     * @param id the database ID to be used as an index
     * @param val the literal or resources to be stored
     */
    public void put(IDBID id, Object val);

    /**
     * Retreive an object from the cache
     * @param id the database ID of the object to be retrieved
     * @return the object or null if it is not in the cache
     */
    public Object get(IDBID id);

    /**
     * Set a threshold for the cache size in terms of the count of cache entries.
     * For literals a storage limit rather than a count might be more useful but
     * counts are easier, more general and sufficient for the current use.
     *
     * @param threshold the cache size limit, use 0 for no cache, -1 for
     * unlimited cache growth; any other number indicates the number of cache entries
     */
    public void setLimit(int threshold);

    /**
     * Return the current threshold limit for the cache size.
     */
    public int getLimit();
}