/* -*-Java-*-
*******************************************************************
*
* File:         IConstraints.java
* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/IConstraints.java,v 1.2 2001/11/26 18:20:09 der Exp $
* Author:       Dave Reynolds, HPLabs, Bristol
* Created:      1/11/01
* Modified:     $Date: 2001/11/26 18:20:09 $
* Status:       Experimental (Do Not Distribute)
*
* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.
* [See end of file]
*********************************************************************/

//=======================================================================
// Package
package com.hp.hpl.mesa.rdf.jena.rdb;

//=======================================================================
// Imports
import com.hp.hpl.mesa.rdf.jena.model.*;
import java.util.Iterator;

//=======================================================================
/**
* Interface for objects which apply a set of additional constraints
* to a listStatements operation. These are generated by factory methods
* on the driver {@link IRDBDriver#createConstraints} since different database layouts may require different
* constraint implementations.
*
* @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>
* @version $Revision: 1.2 $ on $Date: 2001/11/26 18:20:09 $
*/

public interface IConstraints {
    /**
     * Add a constraint that the subject field should match the given resource
     */
    public IConstraints addSubjectConstraint(Resource r) throws RDFException;

    /**
     * Add a constraint that the property field should match the given resource
     */
    public IConstraints addPropertyConstraint(Resource r) throws RDFException;

    /**
     * Add a constraint that the object field should match the given resource
     */
    public IConstraints addObjectConstraint(Resource r) throws RDFException;

    /**
     * Add a constraint that the object field should match the given literal exactly
     */
    public IConstraints addObjectConstraint(Literal l) throws RDFException;

    /**
     * Add a constraint that the object field should match the given string literal
     * using the sql comparison operator op - such as "LIKE" or "NOT LIKE"
     */
    public IConstraints addStringConstraint(String op, String constraint);

    /**
     * Add a constraint that the object field should match the given integer literal
     * using the sql comparison operator op - examples include "&ge;" and "&le;=".
     */
    public IConstraints addIntConstraint(String op, int constraint);

    /**
     * Clear all constraints
     */
    public IConstraints reset();

    /**
     * Execute the query implied by this set of constraints against the statements
     * in the given model.
     */
    public ResultSetIterator runQuery(IRDBDriver driver, ModelRDB m) throws RDFException;

}

/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

