/* -*-Java-*-
*******************************************************************
*
* File:         PropertyImplRDB.java
* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/PropertyImplRDB.java,v 1.2 2001/11/26 18:20:10 der Exp $
* Author:       Dave Reynolds, HPLabs, Bristol
* Created:      7/9/01
* Modified:     $Date: 2001/11/26 18:20:10 $
* Status:       Experimental (Do Not Distribute)
*
* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.
* [See end of file]
*********************************************************************/

//=======================================================================
// Package
package com.hp.hpl.mesa.rdf.jena.rdb;

//=======================================================================
// Imports
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDF;
import java.lang.Integer;
import java.lang.NumberFormatException;

//=======================================================================
/**
* A variation on the default Property implementation that adds a unique
* database ID field. This could be hash generated from the string given
* a sufficiently unique hash but currently is assigned outside this class,
* typically as a generated ID in a database table.
*
* @todo This version is just a cut/paste of PropertyImpl, should refactor inheritance
* hierarchy to avoid this.
*
* @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>
* @version $Revision: 1.2 $ on $Date: 2001/11/26 18:20:10 $
*/

public class PropertyImplRDB extends ResourceImplRDB implements Property {
    /**
     * Additional constructor beyond standard PropertyImpl set, used to
     * create a copy of a property attached to a new model.
     */
    public PropertyImplRDB(Property p, Model m) throws RDFException {
        this(p.getNameSpace(), p.getLocalName(), p.getOrdinal(), m);
    }


//=======================================================================
// Rest is purely a cut/past of PropertyImpl

    protected int    ordinal   = 0;

    /** Creates new PropertyImpl */
    public PropertyImplRDB(String uri) throws RDFException {
        super(uri);
        if (localName == null || localName.equals("")) {
            throw new RDFException(RDFException.INVALIDPROPERTYURI);
        }
        checkOrdinal();
    }

    public PropertyImplRDB(String nameSpace, String localName)
      throws RDFException {
        super(nameSpace, localName);
        checkOrdinal();
    }

    public PropertyImplRDB(String uri, Model m) throws RDFException {
        super(uri, m);
        if ((localName == null) || localName.equals("")) {
            throw new RDFException(RDFException.INVALIDPROPERTYURI);
        }
        checkOrdinal();
    }

    public PropertyImplRDB(String nameSpace, String localName, Model m)
      throws RDFException {
        super(nameSpace, localName, m);
        checkOrdinal();
    }

    public PropertyImplRDB(String nameSpace,
                        String localName,
                        int ordinal,
                        Model m) throws RDFException {
        super(nameSpace, localName, m);
        this.ordinal = ordinal;
    }

    public String getNameSpace() {
      return nameSpace;
    }
    public String getLocalName() {
      return localName;
    }

    public int getOrdinal() {
        return ordinal;
    }

    protected void checkOrdinal()
    {
        char c;

        // check for an rdf:_xxx property

        if (   localName.charAt(0) == '_' && nameSpace.equals(RDF.getURI())
            && nameSpace.equals(RDF.getURI())
            && localName.length() > 1) {
            for (int i=1; i<localName.length(); i++) {
                c = localName.charAt(i);
                if (c < '0'  || c > '9') {
                    return;
                }
            }
            try {
              ordinal = Integer.parseInt(localName.substring(1));
            } catch (NumberFormatException e) {
                ErrorHelper.logInternalError(this.getClass().getName(), 1);
            }
        }
    }

    public Resource port(Model m) throws RDFException {
        if (model == m) {
            return this;
        } else {
            PropertyImplRDB p = new PropertyImplRDB(nameSpace, localName, ordinal, m);
            p.id = id;
            return p;
        }
    }

} // End class

/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

