/* -*-Java-*-
*******************************************************************
*
* File:         ResourceImplRDB.java
* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/ResourceImplRDB.java,v 1.3 2002/07/01 15:20:21 der Exp $
* Author:       Dave Reynolds, HPLabs, Bristol
* Created:      28/8/01
* Modified:     $Date: 2002/07/01 15:20:21 $
* Status:       Experimental (Do Not Distribute)
*
* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.
* [See end of file]
*********************************************************************/

//=======================================================================
// Package
package com.hp.hpl.mesa.rdf.jena.rdb;

//=======================================================================
// Imports
import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import java.io.*;

//=======================================================================
/**
* A variation on the default Resource implementation that adds a unique
* database ID field. This could be hash generated from the string given
* a sufficiently unique hash but currently is assigned outside this class,
* typically as a generated ID in a database table.
*
* @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>
* @version $Revision: 1.3 $ on $Date: 2002/07/01 15:20:21 $
*/

public class ResourceImplRDB extends ResourceImpl  implements HasRDBID {

    /** Unique ID for resource */
    protected IDBID m_rdbId = null;

    /** Unique ID for the resource's namespace */
    protected IDBID m_rdbNSId = null;

    /** Create a resource of unknown uri for a model */
    public ResourceImplRDB(Model m)  {
        super(m);
    }

    /** Create a resource of known uri for a model */
    public ResourceImplRDB(String uri, Model m) {
        super(uri, m);
    }

    /** Create a resource of known uri for a model */
    public ResourceImplRDB(String namespace, String localname, Model m) {
        super(namespace, localname, m);
    }

    /** Create a new resource for this model with the same parameters as an existing resource */
    public ResourceImplRDB(Resource r, Model m) {
        super(r, m);
    }

    /**
     * Create an anonymouse resource for a model
     */
    public ResourceImplRDB(AnonId id, Model m) {
        super(id, m);
    }

    /** Create resources of known uri */
    public ResourceImplRDB(String namespace, String localname) {
        super (namespace, localname);
    }

    /** Create resources of known uri */
    public ResourceImplRDB(String uri) {
        super (uri);
    }

    /**
     * Return the unique ID for the resource or -1 for resources
     * which have not yet be assigned an ID.
     */
    public IDBID getRDBId() {
        return m_rdbId;
    }

    /**
     * Set the unique ID for the resource.
     */
    public void setRDBId(IDBID id) {
        m_rdbId = id;
    }

    /**
     * Return the unique ID for the resource's namespace or -1 for resources
     * which have not yet be assigned an ID.
     */
    public IDBID getRDBNSId() {
        return m_rdbNSId;
    }

    /**
     * Set the unique ID for the resource's namespace
     */
    public void setRDBNSId(IDBID id) {
        m_rdbNSId = id;
    }

} // End class

/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

