/* -*-Java-*-
*******************************************************************
*
* File:         SimpleCache.java
* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/SimpleCache.java,v 1.1 2002/08/09 12:12:50 der Exp $
* Author:       Dave Reynolds, HPLabs, Bristol
* Created:      8/8/02
* Modified:     $Date: 2002/08/09 12:12:50 $
* Status:       Experimental (Do Not Distribute)
*
* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.
* [See end of file]
*********************************************************************/

//=======================================================================
// Package
package com.hp.hpl.mesa.rdf.jena.rdb;

//=======================================================================
// Imports
import java.util.*;

//=======================================================================
/**
* Trivial implementation of the generic cache interface used to cache
* literals and resources. This implementation simple flushes the cache
* when the threshold limit is exceeded.
*
* @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>
* @version $Revision: 1.1 $ on $Date: 2002/08/09 12:12:50 $
*/

public class SimpleCache implements ICache {

    /** The cache itself */
    protected Map cache = new HashMap();

    /** The current size limit */
    protected int threshold;

    /** The current number of entries (probably redundant, just use cache.size) */
    protected int count = 0;

    /**
     * Create an empty cache with the given threshold limit.
     *
     * @param threshold the cache size limit, use 0 for no cache, -1 for
     * unlimited cache growth; any other number indicates the number of cache entries
     */
    public SimpleCache(int threshold) {
        this.threshold = threshold;
    }

    /**
     * Add an entry to the cache
     * @param id the database ID to be used as an index
     * @param val the literal or resources to be stored
     */
    public void put(IDBID id, Object val) {
        if (threshold == 0) return;
        if (threshold > 0 && count >= threshold) {
            cache = new HashMap();
            count = 0;
        }
        count++;
        cache.put(id, val);
    }

    /**
     * Retreive an object from the cache
     * @param id the database ID of the object to be retrieved
     * @return the object or null if it is not in the cache
     */
    public Object get(IDBID id) {
        return cache.get(id);
    }

    /**
     * Set a threshold for the cache size in terms of the count of cache entries.
     * For literals a storage limit rather than a count might be more useful but
     * counts are easier, more general and sufficient for the current use.
     *
     * @param threshold the cache size limit, use 0 for no cache, -1 for
     * unlimited cache growth; any other number indicates the number of cache entries
     */
    public void setLimit(int threshold) {
        this.threshold = threshold;
        if (threshold >= 0 && count > threshold) {
            cache = new HashMap();
            count = 0;
        }
    }

    /**
     * Return the current threshold limit for the cache size.
     */
    public int getLimit() {
        return threshold;
    }
}