/* -*-Java-*-

*******************************************************************

*

* File:         TestInterBaseDriver.java

* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/TestDBConnection.java,v 1.3 2002/03/01 14:25:13 der Exp $

* Author:       Dave Reynolds, HPLabs, Bristol

* Created:      28/8/01

* Modified:     $Date: 2002/03/01 14:25:13 $

* Status:       Experimental (Do Not Distribute)

*

* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.

* [See end of file]

*********************************************************************/



//=======================================================================

// Package

package com.hp.hpl.mesa.rdf.jena.rdb;



//=======================================================================

// Imports

import com.hp.hpl.mesa.rdf.jena.rdb.*;

import com.hp.hpl.mesa.rdf.jena.model.*;

import com.hp.hpl.mesa.rdf.jena.common.*;

import java.io.*;

import java.sql.*;

import java.util.*;

import junit.framework.*;



//=======================================================================

/**

 *

 * @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>

 * @version $Revision: 1.3 $ on $Date: 2002/03/01 14:25:13 $

 */

public class TestDBConnection extends TestCase {



    /** Database base name */

    protected String m_dbname;



    /** Database user name */

    protected String m_user;



    /** Database user password key */

    protected String m_password;



    /** The database layout to test */

    protected String m_layout;



    /** The database type to test */

    protected String m_database;



    public TestDBConnection(String name, String layout, String database) {

        super(name);

        m_layout = layout;

        m_database = database;

    }



    public void setUp() throws ClassNotFoundException {

        //Class.forName("interbase.interclient.Driver");

        m_dbname = System.getProperty(TestRDB.DBFULL_PROP);

        m_user =  System.getProperty(TestRDB.DBUSER_PROP, "DER");

        m_password =  System.getProperty(TestRDB.DBPASSWORD_PROP, "key");

    }



    public void tearDown() {

    }



    public void testBasicOperations() throws SQLException, RDFRDBException, ClassNotFoundException {

        IDBConnection dbcon = new DBConnection(m_dbname, m_user, m_password);

        IRDBDriver driver = dbcon.getDriver(m_layout, m_database);

        System.out.println("Driver = " + driver);

        //dbcon.getProperties().list(System.out);

        // Force load of appropriate jdbc driver -- internal hacking, not part of test

        Class.forName(((DriverGenericGeneric)driver).DRIVER_NAME);

        // Make sure we can open a connection

        Connection con = dbcon.getConnection();

        assertNotNull(con);

        dbcon.close();

        // Check it can find an appropriate driver

        assertNotNull(driver);

        String driverName = "com.hp.hpl.mesa.rdf.jena.rdb.Driver" + m_database + m_layout;

        //Obsolete test - we've now generalized the driver loading

        //assertEquals(driverName, driver.getClass().getName());

        // Clean up the database

        if (driver.isDBFormatOK()) {

            driver.cleanDB();

        }

        driver.formatDB();

        assertTrue(driver.isDBFormatOK());

        assertTrue(dbcon.isFormatOK());

        // set some properties

        dbcon.addProperty("test", "foobar");

        // Close down, open as raw connection and test properties

        dbcon.close();

        con = DriverManager.getConnection(m_dbname, m_user, m_password);

        IDBConnection dbconRaw = new DBConnection(con);

        IRDBDriver driverRaw = dbconRaw.getDriver();

        assertTrue(dbconRaw.isFormatOK());

        //assertEquals(driverName, driverRaw.getClass().getName());

        assertEquals(dbconRaw.getProperty(IDBConnection.DRIVER_CLASSNAME), driverRaw.getClass().getName());

        assertEquals(m_layout, dbconRaw.getProperty(IDBConnection.LAYOUT_TYPE));

        assertEquals(m_database, dbconRaw.getProperty(IDBConnection.DATABASE_TYPE));

        assertEquals("foobar", dbconRaw.getProperty("test"));

        driverRaw.close();

        // Now clean up the database

        driver.cleanDB();

        driver.close();

    }



    public static  TestSuite suite(String layout, String database) {

        TestSuite suite = new TestSuite();

        suite.addTest(new TestDBConnection("testBasicOperations", layout, database));

        return suite;

    }



    /** Top level test suite */

    public static void main(String[] args) {

        junit.textui.TestRunner.run(suite("Generic", "Interbase"));

        //junit.swingui.TestRunner.run(TestInterBaseDriver.class);

    }





}



/*

 *  (c) Copyright Hewlett-Packard Company 2000, 2001

 *  All rights reserved.

 *

 * Redistribution and use in source and binary forms, with or without

 * modification, are permitted provided that the following conditions

 * are met:

 * 1. Redistributions of source code must retain the above copyright

 *    notice, this list of conditions and the following disclaimer.

 * 2. Redistributions in binary form must reproduce the above copyright

 *    notice, this list of conditions and the following disclaimer in the

 *    documentation and/or other materials provided with the distribution.

 * 3. The name of the author may not be used to endorse or promote products

 *    derived from this software without specific prior written permission.



 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR

 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES

 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.

 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,

 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT

 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,

 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY

 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT

 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF

 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */