/* -*-Java-*-

*******************************************************************

*

* File:         TestJenaRegression.java

* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/TestJenaRegression.java,v 1.8 2002/07/05 09:11:47 der Exp $

* Author:       Dave Reynolds, HPLabs, Bristol

* Created:      13/9/01

* Modified:     $Date: 2002/07/05 09:11:47 $

* Status:       Experimental (Do Not Distribute)

*

* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.

* [See end of file]

*********************************************************************/



//=======================================================================

// Package

package com.hp.hpl.mesa.rdf.jena.rdb;



//=======================================================================

// Imports

import com.hp.hpl.mesa.rdf.jena.model.*;

import com.hp.hpl.mesa.rdf.jena.common.regression.TestCaseBasic;

import com.hp.hpl.jena.util.Log;

import junit.framework.*;



//=======================================================================

/**

* Run jena regression tests on an RDB-backed model.

* Currently configured for Interbase back ends.

* Configured using system properties:

* <ul>

* <li>jena.rdb.test.dbbase base directory for databases

* <li>jena.rdb.test.user user name for logging in to dbms

* <li>jena.rdb.test.password password for logging in to dbms

* </ul>

*

* @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>

* @version $Revision: 1.8 $ on $Date: 2002/07/05 09:11:47 $

*/



public class TestJenaRegression {



    /**

     * Create a jena regression test which uses RDB databases with the given

     * layout and database config.

     * The baseuri should be the full database path for databases that support

     * multiple models otherwise it should be a base path to which the suite

     * will append "jr1" to "jr4" to open four separate databases. In all cases the

     * database files must already exist.

     * @param layout the name of the layout format to test, e.g. "MMGeneric"

     * @param databaseType the name of the database type to test, e.g. "Postgresql"

     * @param noReformat suppress formatting of the database between tests

     */

    public static TestSuite suite(String layout, String databaseType, boolean noReformat) {

        String dbBase = System.getProperty(TestRDB.DBBASE_PROP);

        String user =  System.getProperty(TestRDB.DBUSER_PROP, "DER");

        String password =  System.getProperty(TestRDB.DBPASSWORD_PROP, "key");

        ConfigTestCaseRDB config = new ConfigTestCaseRDB(dbBase, user, password, layout, databaseType, noReformat);



        TestSuite suite = new TestSuite();

        suite.addTest(new TestCaseRDB("test0", config));

        suite.addTest(new TestCaseRDB("test1", config));

        suite.addTest(new TestCaseRDB("test2", config));

        suite.addTest(new TestCaseRDB("test3", config));

        suite.addTest(new TestCaseRDB("test4", config));

        suite.addTest(new TestCaseRDB("test5", config));

        suite.addTest(new TestCaseRDB("test6", config));

        suite.addTest(new TestCaseRDB("test7", config));

        suite.addTest(new TestCaseRDB("test8", config));

        suite.addTest(new TestCaseRDB("test9", config));

        suite.addTest(new TestCaseRDB("test10", config));

        suite.addTest(new TestCaseRDB("test11", config));

        suite.addTest(new TestCaseRDB("test12", config));

        suite.addTest(new TestCaseRDB("test13", config));

        suite.addTest(new TestCaseRDB("test14", config));

        suite.addTest(new TestCaseRDB("test15", config));

        suite.addTest(new TestCaseRDB("test16", config));

        suite.addTest(new TestCaseRDB("test17", config));

        suite.addTest(new TestCaseRDB("test18", config));

        suite.addTest(new TestCaseRDB("test19", config));

        // if (config.supportsJenaReification)

        //            suite.addTest(new TestCaseRDB("test20", config));

        suite.addTest(new TestCaseRDB("test97", config));



        //suite.addTest(new TestCaseRDB("testModelEquals", config));

        //suite.addTest(new TestCaseRDB("testMatch", config));

        //suite.addTest(new TestCaseRDB("testWriterAndReader", config));

        //suite.addTest(new TestCaseRDB("testNTripleReader", config));

        //suite.addTest(new TestCaseRDB("testWriterInterface", config));

        //suite.addTest(new TestCaseRDB("testReaderInterface", config));



        return suite;

    }



    /** Inner class which provides config information to TestCaseRDB */

    protected static class ConfigTestCaseRDB {

        /** base uri for the test databases*/

        String m_baseuri;



        /** User name for access the databases */

        String m_user;



        /** Password for this user */

        String m_password;



        /** table layout version to test */

        String m_layout;



        /** database type under test */

        String m_databaseType;



        /** flag if this database config supports multiple models per database */

        boolean supportsMultipleModels;



        /** flag if this database config supports jena-style reification */

        boolean supportsJenaReification;



        /** flag if the tearDown code should leave the DB tables intact by doing a manual database cleanup */

        boolean noReformat;



        /** Database connection */

        DBConnection m_dbconn = null;



        /** Create config.

         *  Needs a base uri for the database, user name and login, format and database type to test.

         *  For databases which support multiple models given the whole database uri. For single model

         *  databases give a base uri to which the model names should be appended.

         */

        ConfigTestCaseRDB(String baseuri, String user, String password, String layout, String database) {

            m_baseuri = baseuri;

            m_user = user;

            m_password = password;

            m_layout = layout;

            m_databaseType = database;

            try {

                DBConnection temp = new DBConnection(baseuri, user, password);

                IRDBDriver driver = temp.getDriver(layout, database);

                supportsMultipleModels = driver.supportsMultipleModels();

                supportsJenaReification = driver.supportsJenaReification();

            } catch (RDFRDBException e) {

                supportsMultipleModels = false;

            }

            noReformat = false;

        }



        /** Create config.

         *  Needs a base uri for the database, user name and login, format and database type to test.

         *  For databases which support multiple models given the whole database uri. For single model

         *  databases give a base uri to which the model names should be appended.

         */

        ConfigTestCaseRDB(String baseuri, String user, String password, String layout, String database, boolean noReformat) {

            this(baseuri, user, password, layout, database);

            this.noReformat = noReformat;

        }



        /** Create a model of the given name for this database config */

        ModelRDB createModel(String name) throws RDFException {

            if (supportsMultipleModels) {

                if (m_dbconn == null) {

                    m_dbconn = new DBConnection(System.getProperty(TestRDB.DBFULL_PROP), m_user, m_password);

                    if (!m_dbconn.isFormatOK()) {

                        IRDBDriver driver = m_dbconn.getDriver(m_layout, m_databaseType);

                        driver.formatDB();

                    }

                }

                return ModelRDB.createModel(m_dbconn, name);

            } else {

                DBConnection dbcon = new DBConnection(m_baseuri + name, m_user, m_password);

                return ModelRDB.create(dbcon, m_layout, m_databaseType);

            }

        }



    } /// End of inner class ConfigTestCaseRDB



    /** Adapt the overall jena test suite to use an RDB store */

    protected static class TestCaseRDB extends TestCaseBasic {

        ConfigTestCaseRDB m_config;



        public TestCaseRDB(String name, ConfigTestCaseRDB config) {

            super(name);

            m_config = config;

        }



        // Override set up to create RDB models instead of mem models

        public void setUp() throws RDFException{

            m1 = m_config.createModel("jr1");

            m2 = m_config.createModel("jr2");

            m3 = m_config.createModel("jr3");

            m4 = m_config.createModel("jr4");

        }



        public void tearDown() {

            if (m_config.supportsMultipleModels && !m_config.noReformat) {

                // The brute force clean deletes the entire DB so only need to do it once

                cleanModel((ModelRDB)m1);

                m2.close();

                m3.close();

                m4.close();

                // Close connection so next time it reformats the DB

                try {

                    m_config.m_dbconn.close();

                } catch (java.sql.SQLException e) {

                    System.out.println("Problem during db clean up in regression test");

                }

                m_config.m_dbconn = null;

            } else {

                cleanModel((ModelRDB)m1);

                cleanModel((ModelRDB)m2);

                cleanModel((ModelRDB)m3);

                cleanModel((ModelRDB)m4);

            }

        }



        private void cleanModel(ModelRDB m) {

            try {

                if (m_config.noReformat) {

                    // Do a slow, brute force manual clean to avoid the database getting reformatted

                    // This is needed to supporting checking of legacy formats

                    for (StmtIterator i = m.listStatements(); i.hasNext(); ) {

                        i.next();

                        i.remove();

                    }

                } else {

                    // Turn off messages about deleting tables that aren't there any more.

                    int l = Log.getInstance().getLevel();

                    Log.getInstance().setLevel(Log.SEVERE);

                    IRDBDriver driver = m.getStore().getDriver();

                    driver.cleanDB();

                    driver.close();

                    m.close();

                    Log.getInstance().setLevel(l);

                }

            } catch (RDFException e) {

                assertTrue("Problem clearning up regression databases: " + e, false);

            }

        }



    }  // End of inner class TestCaseRDB



}



/*

 *  (c) Copyright Hewlett-Packard Company 2000, 2001

 *  All rights reserved.

 *

 * Redistribution and use in source and binary forms, with or without

 * modification, are permitted provided that the following conditions

 * are met:

 * 1. Redistributions of source code must retain the above copyright

 *    notice, this list of conditions and the following disclaimer.

 * 2. Redistributions in binary form must reproduce the above copyright

 *    notice, this list of conditions and the following disclaimer in the

 *    documentation and/or other materials provided with the distribution.

 * 3. The name of the author may not be used to endorse or promote products

 *    derived from this software without specific prior written permission.



 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR

 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES

 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.

 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,

 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT

 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,

 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY

 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT

 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF

 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */