/* -*-Java-*-

*******************************************************************

*

* File:         TestStoreRDB.java

* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/TestRDB.java,v 1.7 2002/07/01 15:20:21 der Exp $

* Author:       Dave Reynolds, HPLabs, Bristol

* Created:      7/9/01

* Modified:     $Date: 2002/07/01 15:20:21 $

* Status:       Experimental (Do Not Distribute)

*

* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.

* [See end of file]

*********************************************************************/



//=======================================================================

// Package

package com.hp.hpl.mesa.rdf.jena.rdb;



//=======================================================================

// Imports

import junit.framework.*;



//=======================================================================

/**

 * Overall test harness for running all currently using rdb unit

 * tests. Exports a final test suite to be included in the overall

 * mega test. Configured via java properties:

 * <ul>

 *   <li> <code>jena.rdb.test.dbbase</code> the jdbc uri of the database stem to use, clean databases

 *   with the names "testrdf", "jr" and "jr1" through "jr4" are needed to run the full test set.

 *   <li> <code>jena.rdb.test.dbfull</code> the full jdbc spec for a test database to use, not just a stem,

 *         useful For jdbc drivers which don't allow easy concatenation of names (e.g. Oracle). Note that this

 *         form won't support the full regression test option for non-multi-model configurations.

 *   <li> <code>jena.rdb.test.user</code> the user name to use when opening the database connection. </li>

 *   <li> <code>jena.rdb.test.password</code> the password to use when opening the database connection. </li>

 * </ul>

 *

 * @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>

 * @version $Revision: 1.7 $ on $Date: 2002/07/01 15:20:21 $

 */



public class TestRDB extends TestCase {



    private static String TEST_LAYOUT = "Generic";

    private static String TEST_DB = "Generic";



    static final  String DBBASE_PROP = "jena.rdb.test.dbbase";

    static final  String DBFULL_PROP = "jena.rdb.test.dbfull";

    static final  String DBUSER_PROP = "jena.rdb.test.user";

    static final  String DBPASSWORD_PROP = "jena.rdb.test.password";



    private static boolean fullRegression = false;

    private static boolean legacyRegression = false;



    public TestRDB(String name) {

        super(name);

    }



    /**

     * Return the total test suite

     */

    public static TestSuite suite() {

        TestSuite suite = new TestSuite();

        if (!legacyRegression) {

            suite.addTest(TestDBConnection.suite(TEST_LAYOUT, TEST_DB));

            suite.addTest(TestGenericDriver.suite(TEST_LAYOUT, TEST_DB));

            suite.addTest(TestStoreRDB.suite(TEST_LAYOUT, TEST_DB));

        }

        if (fullRegression | legacyRegression) suite.addTest(TestJenaRegression.suite(TEST_LAYOUT, TEST_DB, legacyRegression));

        return suite;

    }



    /** signal back usage information and exit */

    private static void usage() {

        System.out.println("Usage: : TestRDF layoutType databaseType [full | legacyFull]");

        System.exit(1);

    }



    /**

     * Run the overall rdf test suite in text mode.

     * <br>Usage: TestRDF layoutType databaseType [full | legacyFull]

     * <p>"full" indicates that the tests include the full jena regression test.

     * <p>"legacyFull" skips the RDB specific tests and just runs the main

     * jena regression but does so in a configuration that avoids reformatting the database.

     * Thus if a database is formated according to an earlier jena version this not enables

     * you to test that the current code can still drive the old format.

     */

    public static void main(String[] args) {

        if (args.length < 2 || args.length > 3) {

            usage();

        }

        TEST_LAYOUT = args[0];

        TEST_DB = args[1];

        if (args.length == 3) {

            if (args[2].equalsIgnoreCase("full")) {

                fullRegression = true;

            } else if (args[2].equalsIgnoreCase("legacyFull")) {

                legacyRegression = true;

            } else {

                usage();

            }

        }



        try {

            if (System.getProperty(DBBASE_PROP) == null) {

                if (TEST_DB.equals("Mysql")) {

                    System.setProperty(DBBASE_PROP, "jdbc:mysql://morte.hpl.hp.com/");

                } else if (TEST_DB.equals("Postgresql")) {

                    System.setProperty(DBBASE_PROP, "jdbc:postgresql:");

                } else if (TEST_DB.equals("Interbase")) {

                    System.setProperty(DBBASE_PROP, "jdbc:interbase://reynolds-d-3.hpl.hp.com/c:/der/java/databases/");

                }

            }

            if (System.getProperty(DBFULL_PROP) == null) {

                System.setProperty(DBFULL_PROP, System.getProperty(DBBASE_PROP) + "testrdf");

            }

        } catch (Exception e) {}

        junit.textui.TestRunner.run(suite());

        //junit.swingui.TestRunner.run();

    }



}



/*

 *  (c) Copyright Hewlett-Packard Company 2000, 2001

 *  All rights reserved.

 *

 * Redistribution and use in source and binary forms, with or without

 * modification, are permitted provided that the following conditions

 * are met:

 * 1. Redistributions of source code must retain the above copyright

 *    notice, this list of conditions and the following disclaimer.

 * 2. Redistributions in binary form must reproduce the above copyright

 *    notice, this list of conditions and the following disclaimer in the

 *    documentation and/or other materials provided with the distribution.

 * 3. The name of the author may not be used to endorse or promote products

 *    derived from this software without specific prior written permission.



 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR

 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES

 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.

 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,

 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT

 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,

 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY

 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT

 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF

 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */