/*
 *  (c) Copyright Hewlett-Packard Company 2000 
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Sample1.java demonstrates the creation and navigation of a simple model
 *
 * Created on 28 July 2000, 18:02
 */

package com.hp.hpl.mesa.rdf.jena.sample;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;

import java.io.PrintWriter;

/** Creates and Queries a simple model.
 *
 * <p>  The example is based on creating ratings.
 *      A person has an Hpps:ratings resource which is a bag of
 *      ratings made by that person.  Each rating in the bag consists
 *      of an anonymous resource with two properties, an rdf:value
 *      property which has double as a value and an hpps:about property
 *      whose value is the resource being rated.
 *
 * @author  bwm
 * @version Release='$Name:  $' Revision='$Revision: 1.1 $' Date='$Date: 2001/01/05 14:17:42 $'
 */
public class Sample1 extends Object {
    
      public static void main (String args[]) {
        try {
            sample1(new ModelMem());
        } catch (Exception e) {
            System.out.println("Failed: " + e);
        }
      }

      static void sample1(Model model) throws RDFException {
        
        // other deckarations
        
        NodeIterator ratings = null;
        StmtIterator stmts   = null;
    
        // create an anonymous rating resource with appropriate properties    
       
        Resource rating = 
              model.createResource()
                   .addProperty(RDF.type, H.Rating)
                   .addProperty(RDF.value, 0.75)
                   .addProperty(H.about,
                                model.createResource("uri for album",
                                                       X.Work));
        
        // create a person resource and give it the right type
        // and add a ratings property with a new bag containing
        // the rating created above.
        
        Resource person = 
            model.createResource("URI for person")
                 .addProperty(RDF.type, H.Person)
                 .addProperty(H.ratings, model.createBag()
                                                 .add(rating));
        
        // list all the statements in this model and print them out
        
        stmts = model.listStatements();
        System.out.println("Statements in the model:");
        while (stmts.hasNext()) {
            System.out.println(stmts.next());
        }
        System.out.println();
        
        // get the value of the 1st rating in the bag
        
        double r = person.getProperty(H.ratings)
                         .getProperty(RDF.li(1))
                         .getProperty(RDF.value).getDouble();

        // change the value of the 1st rating in the bag
                        
        person.getProperty(H.ratings)
              .getProperty(RDF.li(1))
              .getProperty(RDF.value).set(r+0.1);
        
        // iterate through all the ratings
        
        
        ratings = person.getProperty(H.ratings)
                        .getBag().iterator();
        System.out.println();
        while (ratings.hasNext()) {
          System.out.println(
            "Rating = " 
            + ((Resource) ratings.next()).getProperty(RDF.value)
                                         .getDouble());
        }
        
        // and finally lets spit out the RDF

        try {
            model.write(new PrintWriter(System.out));
        } catch (Exception e) {
            System.out.println(e);
        }
    }
}