/*
 *  (c) Copyright Hewlett-Packard Company 2000 
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Sample3.java  Load wordnet into a model
 *
 * Created on 19 August 2000, 07:45
 */

package com.hp.hpl.mesa.rdf.jena.sample;

import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;

import java.io.FileReader;
import java.io.Reader;

/** Load Wordnet into a model and count the number of statements and the number
 * of nouns.
 * @author bwm
 * @version Release='$Name:  $' Revision='$Revision: 1.1 $' Date='$Date: 2001/01/05 14:17:42 $'
 */
public class Sample3 extends Object {
    
    // declare base strings for URI's

    static String base = "http://aldabaran.hpl.hp.com/wordnet/";
    static String ssBase = base + "synset/";
    static String wordBase = base + "word/";
    
    static int    ssIdLength = 9;          // length of a wordnet synset id
    
    // declare resources and properties used later
    
    static Resource nounR;
    static Resource verbR;
    static Resource adjectiveR;
    static Resource adjSateliteR;
    static Resource adverbR;
    static Resource unknownR;
    static Property ssTypeP;
    static Property glossP;
    static Property hypernymP;
    static Property entailmentP;
    static Property similarP;
    static Property membMeronymP;
    static Property subsMeronymP;
    static Property partMeronymP;
    static Property causeP;
    static Property verbGroupP;
    static Property valueOfP;
    static Property antonymP;
    static Property seeAlsoP;
    static Property participleP;
    static Property perP;
    static Property frameP;

    /** Load Wordnet into a model and count the number of nouns.
     * @param args args[0] = the directory containing the wordnet files
     */
    public static void main (String args[]) {
        try {
            
            // get directory of wordnet files
            
            String directory = args[0];
        
            // create an empty model
            
            Model model = new ModelMem();
            
            // create resources and properties used below
        
            nounR = model.createResource(base+"noun");
            verbR = model.createResource(base+"verb");
            adjectiveR = model.createResource(base+"adjective");
            adjSateliteR = model.createResource(base+"adjectiveSatelite");
            adverbR = model.createResource(base+"adverb");
            adverbR = model.createResource(base+"unknown");
            ssTypeP = model.createProperty(base, "ssType");
            glossP = model.createProperty(base, "gloss");
            hypernymP = model.createProperty(base, "hypernym");
            entailmentP = model.createProperty(base, "entailment");
            similarP = model.createProperty(base, "similar");
            membMeronymP = model.createProperty(base, "memberMeronym");
            subsMeronymP = model.createProperty(base, "substanceMeronym");
            partMeronymP = model.createProperty(base, "partMeronym");
            causeP = model.createProperty(base, "cause");
            verbGroupP = model.createProperty(base, "verbGroup");
            valueOfP = model.createProperty(base, "valueOf");
            antonymP = model.createProperty(base, "antonym");
            seeAlsoP = model.createProperty(base, "seeAlso");
            participleP = model.createProperty(base, "participle");
            perP = model.createProperty(base, "per");
            frameP = model.createProperty(base, "frame");
            
            // process each wordnet file
        
            readSFile(model, directory + "/wn_s.txt");
            readGFile(model, directory + "/wn_g.txt");
            readSsPairFile(model, directory + "/wn_hyp.txt", "hyp", hypernymP);
            readSsPairFile(model, directory + "/wn_ent.txt", "ent", entailmentP);
            readSsPairFile(model, directory + "/wn_sim.txt", "sim", similarP);
            readSsPairFile(model, directory + "/wn_mm.txt", "mm", membMeronymP);
            readSsPairFile(model, directory + "/wn_ms.txt", "ms", subsMeronymP);
            readSsPairFile(model, directory + "/wn_mp.txt", "mp", partMeronymP);
            readSsPairFile(model, directory + "/wn_cs.txt", "cs", causeP);
            readSsPairFile(model, directory + "/wn_vgp.txt", "vgp", verbGroupP);
            readSsPairFile(model, directory + "/wn_at.txt", "at", valueOfP);
            readWordPairFile(model, directory + "/wn_ant.txt", "ant", antonymP);
            readWordPairFile(model, directory + "/wn_sa.txt", "sa", seeAlsoP);
            readWordPairFile(model, directory + "/wn_ppl.txt", "ppl", participleP);
            readWordPairFile(model, directory + "/wn_per.txt", "per", perP);
            readFrFile(model, directory + "/wn_fr.txt");
            
            System.out.println("Created " + model.size() + " statements.");
            
            ResIterator iter = model.listSubjectsWithProperty(ssTypeP, nounR);
            int count = 0;
            while (iter.hasNext()) {
                count++;
                iter.next();
            }
            
           System.out.println("There are " 
                                      + count + " noun synsets in Wordnet.");

        } catch (Exception e) {
            System.out.println("exception: " + e);
        }
    }
    
    // create a word in a synset
    
    protected static void createSRelation(Model model, String ssId, String word,
                                            int wordNum, char ssType) {
        try {
            Resource ssR;
            Resource wordR;
            Resource ssTypeR;
            
            // test which word type we jave
            
            if (ssType=='n') {
                ssTypeR = nounR;
            } else if (ssType=='v') {
                ssTypeR = verbR;
            } else if (ssType == 'a') {
                ssTypeR = adjectiveR;
            } else if (ssType == 's') {
                ssTypeR = adjSateliteR;
            } else if (ssType == 'r') {
                ssTypeR = adverbR;
            } else {
                System.out.println("unknown ss_type: " + ssType);
                ssTypeR = unknownR;
            }
            
            // set up the model
            
            ssR = model.createSeq(ssBase+ssId)
                       .add(model.createResource(wordBase+word))
                       .addProperty(ssTypeP, ssTypeR);
        } catch (Exception e) {
            System.out.println("exception: " + e);
        }
    }
    
    // add glosses to the model
    
    protected static void createGRelation(Model model,
                                             String ssId, String gloss) {
       try {
            Resource ssR;
            ssR = model.getSeq(ssBase+ssId)
                       .addProperty(glossP, gloss);
        } catch (Exception e) {
            System.out.println(e);
        }
    }
    
    // add a relation between two syssets to the model
    
    protected static void 
       createSsPairRelation(Model model, String ssId1,
                              String ssId2, Property relation) {
        try {
            model.getResource(ssBase+ssId1)
                 .addProperty(relation, model.getProperty(ssBase+ssId2));
        } catch (Exception e) {
            System.out.println("exception: " + e);
        }
    }
    
    // add a realtion between two words or synsets to the model
    
    protected static void
        createWordPairRelation(Model model, String ssId1, int word1,
                                  String ssId2, int word2, Property relation) {
        try {
            Resource subject;
            Resource object;
            
            if (word1 == 0) {
                subject=model.getResource(ssBase+ssId1);
            } else {
                subject = model.getSeq(ssBase+ssId1)
                               .getResource(word1);
            }
            if (word2 == 0) {
                object = model.getResource(ssBase+ssId2);
            } else {
                object = model.getSeq(ssBase+ssId2)
                              .getResource(word2);
            }
            subject.addProperty(relation, object);
        } catch (Exception e) {
            System.out.println(ssId1+":"+word1 + " " + ssId2 + ":" +word2);
            System.out.println(e);
        }
    }  
    
    // add a sentence frame property to the model
    
    protected static void createFrRelation(Model model, String ssId,
                                                   int frame, int word) {
        try {
            if (word ==0) {
                model.getResource(ssBase+ssId)
                     .addProperty(frameP, frame);
            } else {
                model.getSeq(ssBase+ssId)
                     .getResource(word)
                     .addProperty(frameP, frame);
            }
        } catch (Exception e) {
            System.out.println("exception: " + e);
        }
    }
    
    // the rest of this this just parsing the input files
    
    protected static void readSFile(Model model, String filename) {
        try {
            char[] ssId = new char[ssIdLength];
            int wordNum;
            String word;
            char ssType;
            int  count = 1;
            
            FileReader fr = new FileReader(filename);
            while (fr.read() != -1) {           // skip 's'
                fr.read();                      // skip '(';
                fr.read(ssId, 0, ssIdLength);
                fr.read();                      // skip ','
                wordNum = readInt(fr);
                word = readWord(fr);
                ssType = (char) fr.read();
                skipToEndOfLine(fr);            
                createSRelation(model, new String(ssId), word, wordNum, ssType);
                if ((count++ % 10000) == 0) {
                    System.out.println("s: Done " + count);
                }
            }
            fr.close();
            System.out.println("s: completed:");
        } catch (Exception e) {
            System.out.println(e);
        }
    }
    
    protected static void readGFile(Model model, String filename) {
        try {
            char [] ssId = new char[ssIdLength];
            String gloss;
            int count = 1;
            
            FileReader fr = new FileReader(filename);
            while (fr.read() != -1) {                 // skip 'g'
                fr.read();                            // skip '('
                fr.read(ssId, 0, ssIdLength);
                fr.read();                            // skip ','
                gloss = readWord(fr);
                skipToEndOfLine(fr);
                createGRelation(model, new String(ssId), gloss);
                if ((count++ % 10000) == 0) {
                    System.out.println("g: Done " + count);
                }
            }
            fr.close();
            System.out.println("g: completed:");
        } catch (Exception e) {
            System.out.println("exception: " + e);
        }
    }
    
    protected static void readSsPairFile(Model model, String filename,
                                          String operator, Property relation) {
        try {
            int opLength = operator.length();
            int count = 1;
            char [] ssId1 = new char[ssIdLength];
            char [] ssId2 = new char[ssIdLength];
            
            FileReader fr = new FileReader(filename);
            while (fr.read() != -1) {                 // skip first char
                for (int i=0; i<opLength; i++) {
                    fr.read();
                }
                fr.read(ssId1, 0, ssIdLength);
                fr.read();                        // skip ','
                fr.read(ssId2, 0, ssIdLength);
                skipToEndOfLine(fr);
                createSsPairRelation(model, new String(ssId1),
                                          new String(ssId2), relation);
                if ((count++ % 10000) == 0) {
                    System.out.println(operator + ": Done " + count);
                }
            }
            fr.close();
            System.out.println(operator + ": completed:");
        } catch (Exception e) {
            System.out.println(e);
        }
    }
    
    protected static void readWordPairFile(Model model, String filename,
                                           String operator, Property relation) {
        try {
            int opLength = operator.length();
            int count = 1;
            char [] ssId1 = new char[ssIdLength];
            char [] ssId2 = new char[ssIdLength];
            int word1;
            int word2;
            
            FileReader fr = new FileReader(filename);
            while (fr.read() != -1) {                 // skip first char
                for (int i=0; i<opLength; i++) {
                    fr.read();
                }
                fr.read(ssId1, 0, ssIdLength);
                fr.read();                        // skip ','
                word1 = readInt(fr);
                fr.read(ssId2, 0, ssIdLength);
                fr.read();                        // skip ','
                word2 = readInt(fr);
                skipToEndOfLine(fr);
                createWordPairRelation(model, new String(ssId1), word1,
                                        new String(ssId2), word2, relation);
                if ((count++ % 10000) == 0) {
                    System.out.println(operator + ": Done " + count);
                }
            }
            fr.close();
            System.out.println(operator + ": completed:");
        } catch (Exception e) {
            System.out.println("exception: " + e);
        }
    }
    
    protected static void readFrFile(Model model, String filename) {
        try {
            char [] ssId = new char[ssIdLength];
            int frame;
            int word;
            int count = 1;
            
            FileReader fr = new FileReader(filename);
            while (fr.read() != -1) {                 // skip 'f'
                fr.read();                            // skip 'r'
                fr.read();                            // skip '('
                fr.read(ssId, 0, ssIdLength);
                fr.read();                            // skip ','
                frame = readInt(fr);
                word = readInt(fr);
                skipToEndOfLine(fr);
                createFrRelation(model, new String(ssId), frame, word);
                if ((count++ % 10000) == 0) {
                    System.out.println("fr: Done " + count);
                }
            }
            fr.close();
            System.out.println("fr: completed");
        } catch (Exception e) {
            System.out.println("exception: " + e);
        }
    }
    
    protected static int readInt(Reader r) throws java.io.IOException {
        int result = 0;
        char cc;
 
        cc = (char) r.read();
        while ('0' <= cc && cc <= '9') {
            result = result*10 + (int) (cc - '0');
            cc = (char) r.read();
        }
        return result;
    }
    
    protected static String readWord(Reader r) throws java.io.IOException {
        String result = "";
        char cc;
        
        r.read();          // skip initial '
        while (true) {
            cc = (char) r.read();
            if (cc == '\'') {
                cc = (char) r.read();
                if (cc != '\'') {
                    return result;
                }
            }
            result = result + cc;
        }
    } 
    
    protected static void skipToEndOfLine(Reader r) throws java.io.IOException {
        while (((char)r.read()) != '\n') {}
    }
}