/*
 *  (c) Copyright Hewlett-Packard Company 2000 
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Sample5.java
 *
 * Created on 25 August 2000, 11:27
 */

package com.hp.hpl.mesa.rdf.jena.sample;

import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDFS;

import java.io.FileOutputStream;
import java.io.PrintWriter;

/** Another simple demonstration of reification.
 *
 * <p>Reification is the term used when one statement is the subject or object
 *   of another.  It does seem to cause some difficulty in understanding.</p>
 *
 * <p>This implementation takes a very simple view.  A statement is a subclass
 *   of resource, and as such can be the subject or object of another
 *   statement.  Further, it is possible for a statement s2 to be a member
 *   of a model m, to make a statement about another statement s1, without
 *   s1 being a member of m.  This sample code demonstrates this feature.</p>
 * @author bwm
 * @version Release='$Name:  $' Revision='$Revision: 1.1 $' Date='$Date: 2001/01/05 14:17:42 $'
 */
public class Sample5 extends Object {
    /** Another simple reification example, this time provided by Jos De Roo.
     */
    public static void main (String args[]) {
        
        try {
        
            Model m = new ModelMem();        
            Statement s1 = m.createStatement(m.createResource("subject2"),
                                             m.createProperty("foo/pred2"),
                                             "object2");        
            Statement s2 = m.createStatement(m.createResource("subject1"),
                                             m.createProperty("foo/pred1"),
                                             s1);
        
            m.add(s2, m.createProperty("foo/pred3"), "subject3");
            m.write(new PrintWriter(System.out));
        } catch (Exception e) {
            System.out.println(e);
        }
    }
}